@@ -63,6 +63,7 @@ class ClientOptions(options.ConchOptions):
         else:
             sys.exit("Bad escape character '%s'." % esc)
 
+
     def opt_localforward(self, f):
         "Forward local port to remote address (lport:host:port)"
         localPort, remoteHost, remotePort = f.split(':') # doesn't do v6 yet
@@ -70,6 +71,7 @@ class ClientOptions(options.ConchOptions):
         remotePort = int(remotePort)
         self.localForwards.append((localPort, (remoteHost, remotePort)))
 
+
     def opt_remoteforward(self, f):
         """Forward remote port to local address (rport:host:port)"""
         remotePort, connHost, connPort = f.split(':') # doesn't do v6 yet
@@ -77,10 +79,13 @@ class ClientOptions(options.ConchOptions):
         connPort = int(connPort)
         self.remoteForwards.append((remotePort, (connHost, connPort)))
 
+
     def parseArgs(self, host, *command):
         self['host'] = host
         self['command'] = ' '.join(command)
 
+
+
 # Rest of code in "run"
 options = None
 conn = None
@@ -115,7 +120,7 @@ def run():
             if options['logfile'] == '-':
                 f = sys.stdout
             else:
-                f = file(options['logfile'], 'a+')
+                f = open(options['logfile'], 'a+')
         else:
             f = sys.stderr
         realout = sys.stdout
@@ -146,6 +151,8 @@ def run():
         print('Connection to %s closed.' % options['host'])
     sys.exit(exitStatus)
 
+
+
 def handleError():
     from twisted.python import failure
     global exitStatus
@@ -154,11 +161,15 @@ def handleError():
     log.err(failure.Failure())
     raise
 
+
+
 def _stopReactor():
     try:
         reactor.stop()
     except: pass
 
+
+
 def doConnect():
 #    log.deferr = handleError # HACK
     if '@' in options['host']:
@@ -175,14 +186,21 @@ def doConnect():
     host = options['host']
     port = options['port']
     vhk = default.verifyHostKey
+    if not options['host-key-algorithms']:
+        options['host-key-algorithms'] = default.getHostKeyAlgorithms(
+                                             host, options)
     uao = default.SSHUserAuthClient(options['user'], options, SSHConnection())
     connect.connect(host, port, options, vhk, uao).addErrback(_ebExit)
 
+
+
 def _ebExit(f):
     global exitStatus
     exitStatus = "conch: exiting with error %s" % f
     reactor.callLater(0.1, _stopReactor)
 
+
+
 def onConnect():
 #    if keyAgent and options['agent']:
 #        cc = protocol.ClientCreator(reactor, SSHAgentForwardingLocal, conn)
@@ -216,20 +234,28 @@ def onConnect():
                 if e.errno != errno.EBADF:
                     raise
 
+
+
 def reConnect():
     beforeShutdown()
     conn.transport.transport.loseConnection()
 
+
+
 def beforeShutdown():
     remoteForwards = options.remoteForwards
     for remotePort, hostport in remoteForwards:
         log.msg('cancelling %s:%s' % (remotePort, hostport))
         conn.cancelRemoteForwarding(remotePort)
 
+
+
 def stopConnection():
     if not options['reconnect']:
         reactor.callLater(0.1, _stopReactor)
 
+
+
 class _KeepAlive:
 
     def __init__(self, conn):
@@ -238,22 +264,27 @@ class _KeepAlive:
         self.lc = task.LoopingCall(self.sendGlobal)
         self.lc.start(300)
 
+
     def sendGlobal(self):
-        d = self.conn.sendGlobalRequest("conch-keep-alive@twistedmatrix.com",
-                "", wantReply = 1)
+        d = self.conn.sendGlobalRequest(b"conch-keep-alive@twistedmatrix.com",
+                b"", wantReply = 1)
         d.addBoth(self._cbGlobal)
         self.globalTimeout = reactor.callLater(30, self._ebGlobal)
 
+
     def _cbGlobal(self, res):
         if self.globalTimeout:
             self.globalTimeout.cancel()
             self.globalTimeout = None
 
+
     def _ebGlobal(self):
         if self.globalTimeout:
             self.globalTimeout = None
             self.conn.transport.loseConnection()
 
+
+
 class SSHConnection(connection.SSHConnection):
     def serviceStarted(self):
         global conn
@@ -266,6 +297,7 @@ class SSHConnection(connection.SSHConnection):
             del self.__class__.cancelRemoteForwarding
         onConnect()
 
+
     def serviceStopped(self):
         lf = self.localForwards
         self.localForwards = []
@@ -273,26 +305,30 @@ class SSHConnection(connection.SSHConnection):
             s.loseConnection()
         stopConnection()
 
+
     def requestRemoteForwarding(self, remotePort, hostport):
         data = forwarding.packGlobal_tcpip_forward(('0.0.0.0', remotePort))
-        d = self.sendGlobalRequest('tcpip-forward', data,
+        d = self.sendGlobalRequest(b'tcpip-forward', data,
                                    wantReply=1)
         log.msg('requesting remote forwarding %s:%s' %(remotePort, hostport))
         d.addCallback(self._cbRemoteForwarding, remotePort, hostport)
         d.addErrback(self._ebRemoteForwarding, remotePort, hostport)
 
+
     def _cbRemoteForwarding(self, result, remotePort, hostport):
         log.msg('accepted remote forwarding %s:%s' % (remotePort, hostport))
         self.remoteForwards[remotePort] = hostport
         log.msg(repr(self.remoteForwards))
 
+
     def _ebRemoteForwarding(self, f, remotePort, hostport):
         log.msg('remote forwarding %s:%s failed' % (remotePort, hostport))
         log.msg(f)
 
+
     def cancelRemoteForwarding(self, remotePort):
         data = forwarding.packGlobal_tcpip_forward(('0.0.0.0', remotePort))
-        self.sendGlobalRequest('cancel-tcpip-forward', data)
+        self.sendGlobalRequest(b'cancel-tcpip-forward', data)
         log.msg('cancelling remote forwarding %s' % remotePort)
         try:
             del self.remoteForwards[remotePort]
@@ -300,6 +336,7 @@ class SSHConnection(connection.SSHConnection):
             pass
         log.msg(repr(self.remoteForwards))
 
+
     def channel_forwarded_tcpip(self, windowSize, maxPacket, data):
         log.msg('%s %s' % ('FTCP', repr(data)))
         remoteHP, origHP = forwarding.unpackOpen_forwarded_tcpip(data)
@@ -333,14 +370,16 @@ class SSHConnection(connection.SSHConnection):
             # because of the unix thing
             self.__class__.__bases__[0].channelClosed(self, channel)
 
+
+
 class SSHSession(channel.SSHChannel):
 
-    name = 'session'
+    name = b'session'
 
     def channelOpen(self, foo):
         log.msg('session %s open' % self.id)
         if options['agent']:
-            d = self.conn.sendRequest(self, 'auth-agent-req@openssh.com', '', wantReply=1)
+            d = self.conn.sendRequest(self, b'auth-agent-req@openssh.com', b'', wantReply=1)
             d.addBoth(lambda x:log.msg(x))
         if options['noshell']: return
         if (options['command'] and options['tty']) or not options['notty']:
@@ -355,7 +394,7 @@ class SSHSession(channel.SSHChannel):
         self.stdio = stdio.StandardIO(c)
         fd = 0
         if options['subsystem']:
-            self.conn.sendRequest(self, 'subsystem', \
+            self.conn.sendRequest(self, b'subsystem', \
                 common.NS(options['command']))
         elif options['command']:
             if options['tty']:
@@ -363,9 +402,9 @@ class SSHSession(channel.SSHChannel):
                 winsz = fcntl.ioctl(fd, tty.TIOCGWINSZ, '12345678')
                 winSize = struct.unpack('4H', winsz)
                 ptyReqData = session.packRequest_pty_req(term, winSize, '')
-                self.conn.sendRequest(self, 'pty-req', ptyReqData)
+                self.conn.sendRequest(self, b'pty-req', ptyReqData)
                 signal.signal(signal.SIGWINCH, self._windowResized)
-            self.conn.sendRequest(self, 'exec', \
+            self.conn.sendRequest(self, b'exec', \
                 common.NS(options['command']))
         else:
             if not options['notty']:
@@ -373,12 +412,13 @@ class SSHSession(channel.SSHChannel):
                 winsz = fcntl.ioctl(fd, tty.TIOCGWINSZ, '12345678')
                 winSize = struct.unpack('4H', winsz)
                 ptyReqData = session.packRequest_pty_req(term, winSize, '')
-                self.conn.sendRequest(self, 'pty-req', ptyReqData)
+                self.conn.sendRequest(self, b'pty-req', ptyReqData)
                 signal.signal(signal.SIGWINCH, self._windowResized)
-            self.conn.sendRequest(self, 'shell', '')
+            self.conn.sendRequest(self, b'shell', b'')
             #if hasattr(conn.transport, 'transport'):
             #    conn.transport.transport.setTcpNoDelay(1)
 
+
     def handleInput(self, char):
         #log.msg('handling %s' % repr(char))
         if char in ('\n', '\r'):
@@ -417,51 +457,64 @@ class SSHSession(channel.SSHChannel):
             self.escapeMode = 0
             self.write(char)
 
+
     def dataReceived(self, data):
         self.stdio.write(data)
 
+
     def extReceived(self, t, data):
         if t==connection.EXTENDED_DATA_STDERR:
             log.msg('got %s stderr data' % len(data))
             sys.stderr.write(data)
 
+
     def eofReceived(self):
         log.msg('got eof')
         self.stdio.loseWriteConnection()
 
+
     def closeReceived(self):
         log.msg('remote side closed %s' % self)
         self.conn.sendClose(self)
 
+
     def closed(self):
         global old
         log.msg('closed %s' % self)
         log.msg(repr(self.conn.channels))
 
+
     def request_exit_status(self, data):
         global exitStatus
         exitStatus = int(struct.unpack('>L', data)[0])
         log.msg('exit status: %s' % exitStatus)
 
+
     def sendEOF(self):
         self.conn.sendEOF(self)
 
+
     def stopWriting(self):
         self.stdio.pauseProducing()
 
+
     def startWriting(self):
         self.stdio.resumeProducing()
 
+
     def _windowResized(self, *args):
         winsz = fcntl.ioctl(0, tty.TIOCGWINSZ, '12345678')
         winSize = struct.unpack('4H', winsz)
         newSize = winSize[1], winSize[0], winSize[2], winSize[3]
-        self.conn.sendRequest(self, 'window-change', struct.pack('!4L', *newSize))
+        self.conn.sendRequest(self, b'window-change', struct.pack('!4L', *newSize))
+
 
 
 class SSHListenClientForwardingChannel(forwarding.SSHListenClientForwardingChannel): pass
 class SSHConnectForwardingChannel(forwarding.SSHConnectForwardingChannel): pass
 
+
+
 def _leaveRawMode():
     global _inRawMode
     if not _inRawMode:
@@ -470,6 +523,7 @@ def _leaveRawMode():
     tty.tcsetattr(fd, tty.TCSANOW, _savedRawMode)
     _inRawMode = 0
 
+
 def _enterRawMode():
     global _inRawMode, _savedRawMode
     if _inRawMode:
@@ -505,6 +559,8 @@ def _enterRawMode():
         #tty.setraw(fd)
         _inRawMode = 1
 
+
+
 if __name__ == '__main__':
     run()
 
