@@ -27,9 +27,9 @@ import inspect
 import os
 import platform
 import socket
-import string
 import struct
 import sys
+import tokenize
 from types import MethodType as _MethodType
 
 from io import TextIOBase, IOBase
@@ -40,11 +40,23 @@ if sys.version_info < (3, 0):
 else:
     _PY3 = True
 
+if sys.version_info >= (3, 4, 0):
+    _PY34PLUS = True
+else:
+    _PY34PLUS = False
+
+if sys.version_info >= (3, 5, 0):
+    _PY35PLUS = True
+else:
+    _PY35PLUS = False
+
 if platform.python_implementation() == 'PyPy':
     _PYPY = True
 else:
     _PYPY = False
 
+
+
 def _shouldEnableNewStyle():
     """
     Returns whether or not we should enable the new-style conversion of
@@ -85,14 +97,27 @@ def currentframe(n=0):
 
 
 def inet_pton(af, addr):
+    """
+    Emulator of L{socket.inet_pton}.
+
+    @param af: An address family to parse; C{socket.AF_INET} or
+        C{socket.AF_INET6}.
+    @type af: L{int}
+
+    @param addr: An address.
+    @type addr: native L{str}
+
+    @return: The binary packed version of the passed address.
+    @rtype: L{bytes}
+    """
+    if not addr:
+        raise ValueError("illegal IP address string passed to inet_pton")
     if af == socket.AF_INET:
         return socket.inet_aton(addr)
     elif af == getattr(socket, 'AF_INET6', 'AF_INET6'):
-        illegalChars = [x for x in addr if x not in string.hexdigits + ':.']
-        if illegalChars:
-            raise ValueError("Illegal characters: %r" %
-                             (''.join(illegalChars),))
-
+        if '%' in addr and (addr.count('%') > 1 or addr.index("%") == 0):
+            raise ValueError("illegal IP address string passed to inet_pton")
+        addr = addr.split('%')[0]
         parts = addr.split(':')
         elided = parts.count('')
         ipv4Component = '.' in parts[-1]
@@ -132,6 +157,8 @@ def inet_pton(af, addr):
     else:
         raise socket.error(97, 'Address family not supported by protocol')
 
+
+
 def inet_ntop(af, addr):
     if af == socket.AF_INET:
         return socket.inet_ntoa(addr)
@@ -213,7 +240,7 @@ def execfile(filename, globals, locals=None):
     """
     if locals is None:
         locals = globals
-    with open(filename, "rbU") as fin:
+    with open(filename, "rb") as fin:
         source = fin.read()
     code = compile(source, filename, "exec")
     exec(code, globals, locals)
@@ -251,6 +278,7 @@ def comparable(klass):
     if not _PY3:
         return klass
 
+
     def __eq__(self, other):
         c = self.__cmp__(other)
         if c is NotImplemented:
@@ -505,7 +533,6 @@ else:
     def intToBytes(i):
         return b"%d" % i
 
-
     lazyByteSlice = buffer
 
     def networkString(s):
@@ -591,24 +618,30 @@ if _PY3:
     def iteritems(d):
         return d.items()
 
+
     def itervalues(d):
         return d.values()
 
+
     def items(d):
         return list(d.items())
 
+    range = range
     xrange = range
     izip = zip
 else:
     def iteritems(d):
         return d.iteritems()
 
+
     def itervalues(d):
         return d.itervalues()
 
+
     def items(d):
         return d.items()
 
+    range = xrange
     xrange = xrange
     from itertools import izip
     izip # shh pyflakes
@@ -761,28 +794,6 @@ def _coercedUnicode(s):
 
 
 
-def _maybeMBCS(s):
-    """
-    Convert the string C{s} to a L{unicode} string, if required.
-
-    @param s: The string to convert.
-    @type s: L{bytes} or L{unicode}
-
-    @return: The string, decoded using MBCS if needed.
-    @rtype: L{unicode}
-
-    @raises UnicodeDecodeError: If passed a byte string that cannot be decoded
-        using MBCS.
-    """
-    assert sys.platform == "win32", "only reasonable on Windows"
-    assert type(s) in [bytes, unicode], str(type(s)) + " is not a string"
-
-    if isinstance(s, bytes):
-        return s.decode('mbcs')
-    return s
-
-
-
 if _PY3:
     unichr = chr
     raw_input = input
@@ -814,6 +825,12 @@ def _bytesRepr(bytestring):
         return 'b' + repr(bytestring)
 
 
+if _PY3:
+    _tokenize = tokenize.tokenize
+else:
+    _tokenize = tokenize.generate_tokens
+
+
 
 __all__ = [
     "reraise",
@@ -837,6 +854,7 @@ __all__ = [
     "items",
     "iteritems",
     "itervalues",
+    "range",
     "xrange",
     "urllib_parse",
     "bytesEnviron",
@@ -853,5 +871,5 @@ __all__ = [
     "intern",
     "unichr",
     "raw_input",
-    "_maybeMBCS",
+    "_tokenize"
 ]
