@@ -12,7 +12,7 @@ from zope.interface import implementer
 from zope.interface.verify import verifyObject
 
 from twisted.python import reflect, failure
-from twisted.python.compat import _PY3
+from twisted.python.compat import unichr
 from twisted.python.filepath import FilePath
 from twisted.trial import unittest
 from twisted.internet import reactor
@@ -33,9 +33,9 @@ class ResourceTests(unittest.TestCase):
 
 class SimpleResource(resource.Resource):
     """
-    @ivar _contentType: C{None} or a C{str} giving the value of the
+    @ivar _contentType: L{None} or a C{str} giving the value of the
         I{Content-Type} header in the response this resource will render.  If it
-        is C{None}, no I{Content-Type} header will be set in the response.
+        is L{None}, no I{Content-Type} header will be set in the response.
     """
     def __init__(self, contentType=None):
         resource.Resource.__init__(self)
@@ -55,6 +55,26 @@ class SimpleResource(resource.Resource):
 
 
 
+class ZeroLengthResource(resource.Resource):
+    """
+    A resource that always returns a zero-length response.
+    """
+    def render(self, request):
+        return b''
+
+
+
+class NoContentResource(resource.Resource):
+    """
+    A resource that always returns a 204 No Content response without setting
+    Content-Length.
+    """
+    def render(self, request):
+        request.setResponseCode(http.NO_CONTENT)
+        return b''
+
+
+
 class SiteTest(unittest.TestCase):
     """
     Unit tests for L{server.Site}.
@@ -136,6 +156,32 @@ class SiteTest(unittest.TestCase):
         self.assertIsInstance(session.uid, bytes)
 
 
+    def test_sessionUIDGeneration(self):
+        """
+        L{site.getSession} generates L{Session} objects with distinct UIDs from
+        a secure source of entropy.
+        """
+        site = server.Site(resource.Resource())
+        # Ensure that we _would_ use the unpredictable random source if the
+        # test didn't stub it.
+        self.assertIdentical(site._entropy, os.urandom)
+
+        def predictableEntropy(n):
+            predictableEntropy.x += 1
+            return (unichr(predictableEntropy.x) * n).encode("charmap")
+        predictableEntropy.x = 0
+        self.patch(site, "_entropy", predictableEntropy)
+        a = self.getAutoExpiringSession(site)
+        b = self.getAutoExpiringSession(site)
+        self.assertEqual(a.uid, b"01" * 0x20)
+        self.assertEqual(b.uid, b"02" * 0x20)
+        # This functionality is silly (the value is no longer used in session
+        # generation), but 'counter' was a public attribute since time
+        # immemorial so we should make sure if anyone was using it to get site
+        # metrics or something it keeps working.
+        self.assertEqual(site.counter, 2)
+
+
     def test_getSessionExistent(self):
         """
         L{site.getSession} gets a previously generated session, by its unique
@@ -554,8 +600,10 @@ class RequestTests(unittest.TestCase):
         fail = failure.Failure(Exception("Oh no!"))
         request.processingFailed(fail)
 
-        self.assertNotIn(b"Oh no!", request.transport.getvalue())
-        self.assertIn(b"Processing Failed", request.transport.getvalue())
+        self.assertNotIn(b"Oh no!", request.transport.written.getvalue())
+        self.assertIn(
+            b"Processing Failed", request.transport.written.getvalue()
+        )
 
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
@@ -574,7 +622,7 @@ class RequestTests(unittest.TestCase):
         fail = failure.Failure(Exception("Oh no!"))
         request.processingFailed(fail)
 
-        self.assertIn(b"Oh no!", request.transport.getvalue())
+        self.assertIn(b"Oh no!", request.transport.written.getvalue())
 
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
@@ -594,19 +642,193 @@ class RequestTests(unittest.TestCase):
         fail = failure.Failure(Exception(u"\u2603"))
         request.processingFailed(fail)
 
-        self.assertIn(b"&#9731;", request.transport.getvalue())
+        self.assertIn(b"&#9731;", request.transport.written.getvalue())
+
+        # On some platforms, we get a UnicodeError when trying to
+        # display the Failure with twisted.python.log because
+        # the default encoding cannot display u"\u2603".  Windows for example
+        # uses a default encodig of cp437 which does not support u"\u2603".
+        self.flushLoggedErrors(UnicodeError)
 
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
         self.assertEqual(1, len(self.flushLoggedErrors()))
 
 
+    def test_sessionDifferentFromSecureSession(self):
+        """
+        L{Request.session} and L{Request.secure_session} should be two separate
+        sessions with unique ids and different cookies.
+        """
+        d = DummyChannel()
+        d.transport = DummyChannel.SSL()
+        request = server.Request(d, 1)
+        request.site = server.Site(resource.Resource())
+        request.sitepath = []
+        secureSession = request.getSession()
+        self.assertIsNotNone(secureSession)
+        self.addCleanup(secureSession.expire)
+        self.assertEqual(request.cookies[0].split(b"=")[0],
+                         b"TWISTED_SECURE_SESSION")
+        session = request.getSession(forceNotSecure=True)
+        self.assertIsNotNone(session)
+        self.assertEqual(request.cookies[1].split(b"=")[0], b"TWISTED_SESSION")
+        self.addCleanup(session.expire)
+        self.assertNotEqual(session.uid, secureSession.uid)
+
+
+    def test_sessionAttribute(self):
+        """
+        On a L{Request}, the C{session} attribute retrieves the associated
+        L{Session} only if it has been initialized.  If the request is secure,
+        it retrieves the secure session.
+        """
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        d.transport = DummyChannel.SSL()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+        self.assertIs(request.session, None)
+        insecureSession = request.getSession(forceNotSecure=True)
+        self.addCleanup(insecureSession.expire)
+        self.assertIs(request.session, None)
+        secureSession = request.getSession()
+        self.addCleanup(secureSession.expire)
+        self.assertIsNot(secureSession, None)
+        self.assertIsNot(secureSession, insecureSession)
+        self.assertIs(request.session, secureSession)
+
+
+    def test_sessionCaching(self):
+        """
+        L{Request.getSession} creates the session object only once per request;
+        if it is called twice it returns the identical result.
+        """
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+        session1 = request.getSession()
+        self.addCleanup(session1.expire)
+        session2 = request.getSession()
+        self.assertIs(session1, session2)
+
+
+    def test_retrieveExistingSession(self):
+        """
+        L{Request.getSession} retrieves an existing session if the relevant
+        cookie is set in the incoming request.
+        """
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+        mySession = server.Session(b"special-id", site)
+        site.sessions[mySession.uid] = mySession
+        request.received_cookies[b'TWISTED_SESSION'] = mySession.uid
+        self.assertIs(request.getSession(), mySession)
+
+
+    def test_retrieveNonExistentSession(self):
+        """
+        L{Request.getSession} generates a new session if the relevant cookie is
+        set in the incoming request.
+        """
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+        request.received_cookies[b'TWISTED_SESSION'] = b"does-not-exist"
+        session = request.getSession()
+        self.assertIsNotNone(session)
+        self.addCleanup(session.expire)
+        self.assertTrue(request.cookies[0].startswith(b'TWISTED_SESSION='))
+        # It should be a new session ID.
+        self.assertNotIn(b"does-not-exist", request.cookies[0])
+
+
+    def test_OPTIONSStar(self):
+        """
+        L{Request} handles OPTIONS * requests by doing a fast-path return of
+        200 OK.
+        """
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.setHost(b'example.com', 80)
+        request.gotLength(0)
+        request.requestReceived(b'OPTIONS', b'*', b'HTTP/1.1')
+
+        response = d.transport.written.getvalue()
+        self.assertTrue(response.startswith(b'HTTP/1.1 200 OK'))
+        self.assertIn(b'Content-Length: 0\r\n', response)
+
+
+    def test_rejectNonOPTIONSStar(self):
+        """
+        L{Request} handles any non-OPTIONS verb requesting the * path by doing
+        a fast-return 405 Method Not Allowed, indicating only the support for
+        OPTIONS.
+        """
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.setHost(b'example.com', 80)
+        request.gotLength(0)
+        request.requestReceived(b'GET', b'*', b'HTTP/1.1')
+
+        response = d.transport.written.getvalue()
+        self.assertTrue(
+            response.startswith(b'HTTP/1.1 405 Method Not Allowed')
+        )
+        self.assertIn(b'Content-Length: 0\r\n', response)
+        self.assertIn(b'Allow: OPTIONS\r\n', response)
+
+
+    def test_noDefaultContentTypeOnZeroLengthResponse(self):
+        """
+        Responses with no length do not have a default content-type applied.
+        """
+        resrc = ZeroLengthResource()
+        resrc.putChild(b'', resrc)
+        site = server.Site(resrc)
+        d = DummyChannel()
+        d.site = site
+        request = server.Request(d, 1)
+        request.site = site
+        request.setHost(b'example.com', 80)
+        request.gotLength(0)
+        request.requestReceived(b'GET', b'/', b'HTTP/1.1')
+
+        self.assertNotIn(
+            b'content-type', request.transport.written.getvalue().lower()
+        )
+
+
+    def test_noDefaultContentTypeOn204Response(self):
+        """
+        Responses with a 204 status code have no default content-type applied.
+        """
+        resrc = NoContentResource()
+        resrc.putChild(b'', resrc)
+        site = server.Site(resrc)
+        d = DummyChannel()
+        d.site = site
+        request = server.Request(d, 1)
+        request.site = site
+        request.setHost(b'example.com', 80)
+        request.gotLength(0)
+        request.requestReceived(b'GET', b'/', b'HTTP/1.1')
+
+        response = request.transport.written.getvalue()
+        self.assertTrue(response.startswith(b'HTTP/1.1 204 No Content\r\n'))
+        self.assertNotIn(b'content-type', response.lower())
+
+
 
 class GzipEncoderTests(unittest.TestCase):
-
-    if _PY3:
-        skip = "GzipEncoder not ported to Python 3 yet."
-
     def setUp(self):
         self.channel = DummyChannel()
         staticResource = Data(b"Some data", "text/plain")
@@ -823,11 +1045,15 @@ class NewRenderTests(unittest.TestCase):
     def testGoodMethods(self):
         req = self._getReq()
         req.requestReceived(b'GET', b'/newrender', b'HTTP/1.0')
-        self.assertEqual(req.transport.getvalue().splitlines()[-1], b'hi hi')
+        self.assertEqual(
+            req.transport.written.getvalue().splitlines()[-1], b'hi hi'
+        )
 
         req = self._getReq()
         req.requestReceived(b'HEH', b'/newrender', b'HTTP/1.0')
-        self.assertEqual(req.transport.getvalue().splitlines()[-1], b'ho ho')
+        self.assertEqual(
+            req.transport.written.getvalue().splitlines()[-1], b'ho ho'
+        )
 
     def testBadMethods(self):
         req = self._getReq()
@@ -855,7 +1081,9 @@ class NewRenderTests(unittest.TestCase):
         req = self._getReq()
         req.requestReceived(b'HEAD', b'/newrender', b'HTTP/1.0')
         self.assertEqual(req.code, 200)
-        self.assertEqual(-1, req.transport.getvalue().find(b'hi hi'))
+        self.assertEqual(
+            -1, req.transport.written.getvalue().find(b'hi hi')
+        )
 
 
     def test_unsupportedHead(self):
@@ -866,7 +1094,7 @@ class NewRenderTests(unittest.TestCase):
         resource = HeadlessResource()
         req = self._getReq(resource)
         req.requestReceived(b"HEAD", b"/newrender", b"HTTP/1.0")
-        headers, body = req.transport.getvalue().split(b'\r\n\r\n')
+        headers, body = req.transport.written.getvalue().split(b'\r\n\r\n')
         self.assertEqual(req.code, 200)
         self.assertEqual(body, b'')
 
@@ -887,7 +1115,7 @@ class NewRenderTests(unittest.TestCase):
 
         request.requestReceived(b"GET", b"/newrender", b"HTTP/1.0")
 
-        headers, body = request.transport.getvalue().split(b'\r\n\r\n')
+        headers, body = request.transport.written.getvalue().split(b'\r\n\r\n')
         self.assertEqual(request.code, 500)
         expected = [
             '',
@@ -929,10 +1157,6 @@ class AllowedMethodsTests(unittest.TestCase):
     'C{twisted.web.resource._computeAllowedMethods} is provided by a
     default should the subclass not provide the method.
     """
-
-    if _PY3:
-        skip = "Allowed methods functionality not ported to Python 3."
-
     def _getReq(self):
         """
         Generate a dummy request for use by C{_computeAllowedMethod} tests.
@@ -988,7 +1212,7 @@ class AllowedMethodsTests(unittest.TestCase):
         req.requestReceived(b'POST', b'/gettableresource?'
                             b'value=<script>bad', b'HTTP/1.0')
         self.assertEqual(req.code, 405)
-        renderedPage = req.transport.getvalue()
+        renderedPage = req.transport.written.getvalue()
         self.assertNotIn(b"<script>bad", renderedPage)
         self.assertIn(b'&lt;script&gt;bad', renderedPage)
 
@@ -1003,7 +1227,7 @@ class AllowedMethodsTests(unittest.TestCase):
         req = self._getReq()
         req.requestReceived(b'<style>bad', b'/gettableresource', b'HTTP/1.0')
         self.assertEqual(req.code, 501)
-        renderedPage = req.transport.getvalue()
+        renderedPage = req.transport.written.getvalue()
         self.assertNotIn(b"<style>bad", renderedPage)
         self.assertIn(b'&lt;style&gt;bad', renderedPage)
 
@@ -1045,8 +1269,7 @@ class AccessLogTestsMixin(object):
         reactor.advance(1234567890)
 
         logPath = self.mktemp()
-        factory = self.factory(logPath=logPath)
-        factory._reactor = reactor
+        factory = self.factory(logPath=logPath, reactor=reactor)
         factory.startFactory()
 
         try:
@@ -1070,7 +1293,7 @@ class AccessLogTestsMixin(object):
             # Value of the "Referer" header.  Probably incorrectly quoted.
             b'"-" '
             # Value pf the "User-Agent" header.  Probably incorrectly quoted.
-            b'"-"' + self.linesep,
+            b'"-"\n',
             FilePath(logPath).getContent())
 
 
@@ -1096,9 +1319,7 @@ class AccessLogTestsMixin(object):
             factory.stopFactory()
 
         self.assertEqual(
-            # self.linesep is a sad thing.
-            # https://twistedmatrix.com/trac/ticket/6938
-            b"this is a bad log format" + self.linesep,
+            b"this is a bad log format\n",
             FilePath(logPath).getContent())
 
 
@@ -1108,7 +1329,6 @@ class HTTPFactoryAccessLogTests(AccessLogTestsMixin, unittest.TestCase):
     Tests for L{http.HTTPFactory.log}.
     """
     factory = http.HTTPFactory
-    linesep = b"\n"
 
 
 
@@ -1116,10 +1336,6 @@ class SiteAccessLogTests(AccessLogTestsMixin, unittest.TestCase):
     """
     Tests for L{server.Site.log}.
     """
-    if _PY3:
-        skip = "Site not ported to Python 3 yet."
-
-    linesep = os.linesep
 
     def factory(self, *args, **kwargs):
         return server.Site(resource.Resource(), *args, **kwargs)
@@ -1147,7 +1363,7 @@ class CombinedLogFormatterTests(unittest.TestCase):
         reactor.advance(1234567890)
 
         timestamp = http.datetimeToLogString(reactor.seconds())
-        request = DummyRequestForLogTest(http.HTTPFactory())
+        request = DummyRequestForLogTest(http.HTTPFactory(reactor=reactor))
         request.client = IPv4Address("TCP", b"evil x-forwarded-for \x80", 12345)
         request.method = b"POS\x81"
         request.protocol = b"HTTP/1.\x82"
@@ -1188,7 +1404,7 @@ class ProxiedLogFormatterTests(unittest.TestCase):
         reactor.advance(1234567890)
 
         timestamp = http.datetimeToLogString(reactor.seconds())
-        request = DummyRequestForLogTest(http.HTTPFactory())
+        request = DummyRequestForLogTest(http.HTTPFactory(reactor=reactor))
         expected = http.combinedLogFormatter(timestamp, request).replace(
             u"1.2.3.4", u"172.16.1.2")
         request.requestHeaders.setRawHeaders(b"x-forwarded-for", [header])
@@ -1357,3 +1573,29 @@ class ServerAttributesTests(unittest.TestCase):
             warnings[0]['message'],
             ("twisted.web.server.string_date_time was deprecated in Twisted "
              "12.1.0: Please use twisted.web.http.stringToDatetime instead"))
+
+
+
+class ExplicitHTTPFactoryReactor(unittest.TestCase):
+    """
+    L{http.HTTPFactory} accepts explicit reactor selection.
+    """
+
+    def test_explicitReactor(self):
+        """
+        L{http.HTTPFactory.__init__} accepts a reactor argument which is set on
+        L{http.HTTPFactory._reactor}.
+        """
+        reactor = "I am a reactor!"
+        factory = http.HTTPFactory(reactor=reactor)
+        self.assertIs(factory._reactor, reactor)
+
+
+    def test_defaultReactor(self):
+        """
+        Giving no reactor argument to L{http.HTTPFactory.__init__} means it
+        will select the global reactor.
+        """
+        from twisted.internet import reactor
+        factory = http.HTTPFactory()
+        self.assertIs(factory._reactor, reactor)
